/*******************************************************************************
 * Copyright (c) 2013, Perforce Software
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package com.perforce.p4java.extension.operation;

import java.util.Map;

import com.perforce.p4java.extension.command.PerforceDomain;
import com.perforce.p4java.extension.exception.ServerAccessException;
import com.perforce.p4java.extension.exception.ServerConnectionException;
import com.perforce.p4java.extension.exception.ServerErrorException;
import com.perforce.p4java.extension.server.Server;

/**
 * 
 */
public class CounterCommand extends Command<String> {

    private enum Option {

	FORCE("-f"), DELETE("-d"), INCREMENT("-i"), SIDETABLE("-u");

	private String flag;

	private Option(String flag) {
	    this.flag = flag;
	}

	String getFlag() {
	    return flag;
	}

    }

    // escaping for counters: spaces get converted to _ by the server, */#/@ not
    // allowed
    // replace %'s first so that %40 in the original is not converted
    public static String escape(String name) {
	return name.replace("%", "%25").replace("@", "%40").replace("#", "%23")
		.replace(" ", "%20").replace("*", "%2A");
    }

    // replace %'s last (see above)
    private static String unescape(String name) {
	return name.replace("%40", "@").replace("%23", "#").replace("%20", " ")
		.replace("%2A", "*").replace("%25", "%");
    }

    public static String getCounterValue(String name, Server server)
	    throws ServerConnectionException, ServerAccessException,
	    ServerErrorException {
	CounterCommand cmd = new CounterCommand();
	cmd.arguments.add(escape(name));
	return cmd.executeModelRequest(server);
    }

    /**
     * Gets counter on side table. P4D 2012.1.463227 and later only!
     */
    public static String getCounterValueSideTable(String name, Server server)
	    throws ServerConnectionException, ServerAccessException,
	    ServerErrorException {
	CounterCommand cmd = new CounterCommand();
	cmd.arguments.add(Option.SIDETABLE.getFlag());
	cmd.arguments.add(escape(name));
	return cmd.executeModelRequest(server);
    }

    public static String setCounterValue(String name, String value,
	    Server server) throws ServerConnectionException,
	    ServerAccessException, ServerErrorException {
	CounterCommand cmd = new CounterCommand();
	cmd.arguments.add(escape(name));
	cmd.arguments.add(value);
	return cmd.executeModelRequest(server);
    }

    /**
     * Sets counter on side table. P4D 2012.1.463227 and later only!
     */
    public static String setCounterValueSideTable(String name, String value,
	    Server server) throws ServerConnectionException,
	    ServerAccessException, ServerErrorException {
	CounterCommand cmd = new CounterCommand();
	cmd.arguments.add(Option.SIDETABLE.getFlag());
	cmd.arguments.add(escape(name));
	cmd.arguments.add(value);
	return cmd.executeModelRequest(server);
    }

    public static String forceCounterValue(String name, String value,
	    Server server) throws ServerConnectionException,
	    ServerAccessException, ServerErrorException {
	CounterCommand cmd = new CounterCommand();
	cmd.arguments.add(Option.FORCE.getFlag());
	cmd.arguments.add(escape(name));
	cmd.arguments.add(value);
	return cmd.executeModelRequest(server);
    }

    public static String incrementCounter(String name, Server server)
	    throws ServerConnectionException, ServerAccessException,
	    ServerErrorException {
	CounterCommand cmd = new CounterCommand();
	cmd.arguments.add(Option.INCREMENT.getFlag());
	cmd.arguments.add(escape(name));
	return cmd.executeModelRequest(server);
    }

    /**
     * Increments counter on side table. P4D 2012.1.463227 and later only!
     */
    public static String incrementCounterSideTable(String name, Server server)
	    throws ServerConnectionException, ServerAccessException,
	    ServerErrorException {
	CounterCommand cmd = new CounterCommand();
	cmd.arguments.add(Option.SIDETABLE.getFlag());
	cmd.arguments.add(Option.INCREMENT.getFlag());
	cmd.arguments.add(escape(name));
	return cmd.executeModelRequest(server);
    }

    public static String forceIncrementCounter(String name, Server server)
	    throws ServerConnectionException, ServerAccessException,
	    ServerErrorException {
	CounterCommand cmd = new CounterCommand();
	cmd.arguments.add(Option.INCREMENT.getFlag());
	cmd.arguments.add(Option.FORCE.getFlag());
	cmd.arguments.add(escape(name));
	return cmd.executeModelRequest(server);
    }

    public static void deleteCounterSideTable(String name, Server server)
	    throws ServerConnectionException, ServerAccessException,
	    ServerErrorException {
	CounterCommand cmd = new CounterCommand();
	cmd.arguments.add(Option.SIDETABLE.getFlag());
	cmd.arguments.add(Option.DELETE.getFlag());
	cmd.arguments.add(escape(name));
	cmd.executeModelRequest(server);
    }

    private CounterCommand() {
	super(PerforceDomain.COUNTER);
    }

    /**
     * @see com.perforce.p4java.extension.command.Request#getInputString()
     */
    public String getInputString() {
	return null;
    }

    /**
     * @see 
     *      com.perforce.nuapi.operation.Command#process(java.util.Map<java.lang.
     *      String,java.lang.Object>[])
     */
    @Override
    protected String process(Map<String, Object>[] maps) {
	Map<String, Object> counter = maps[0];
	if (counter.containsKey("counter") && counter.containsKey("value")) {
	    return (String) counter.get("value");
	}
	return null;
    }
}
